/* simgrid_config.h - Results of the configure made visible to user code    */

/* Copyright (c) 2009-2014. The SimGrid Team.
 * All rights reserved.                                                     */

/* This program is free software; you can redistribute it and/or modify it
 * under the terms of the license (GNU LGPL) which comes with this package. */

#ifndef SIMGRID_PUBLIC_CONFIG_H
#define SIMGRID_PUBLIC_CONFIG_H

/** Define the version numbers of the used header files. 
    sg_version() can be used to retrieve the version of the dynamic library.
    But actually, if these numbers don't match, SimGrid refuses to start (so you shouldn't have to care about sg_version() yourself) */

#define SIMGRID_VERSION_MAJOR 3
#define SIMGRID_VERSION_MINOR 12
#define SIMGRID_VERSION_PATCH 0

#define SIMGRID_VERSION_STRING "SimGrid version 3.12"

#define SIMGRID_VERSION_BANNER "SIMGRID_VERSION_STRING\nCopyright (c) 2004-2015. The Simgrid Team.\nRelease build at commit 009559e"

/* Version X.Y.Z will get version number XYZ: all digits concatenated without dots
 * (with Y and Z must be on two positions)*/

#define MAKE_SIMGRID_VERSION(major, minor, patch)       \
  (100UL * (100UL * (major) + (minor)) + (patch))
#define SIMGRID_VERSION MAKE_SIMGRID_VERSION(SIMGRID_VERSION_MAJOR, \
                                             SIMGRID_VERSION_MINOR, \
                                             SIMGRID_VERSION_PATCH)

/* take care of DLL usage madness */

#ifdef _XBT_DLL_EXPORT
	#ifndef DLL_EXPORT
		#define DLL_EXPORT
	#endif
#else
	#ifdef _XBT_DLL_STATIC
		#ifndef DLL_STATIC
			#define DLL_STATIC
		#endif
	#else
		#ifndef DLL_EXPORT
			#define DLL_IMPORT
		#endif
	#endif
#endif

/* #undef _XBT_WIN32 */
/* #undef _WIN32 */
/* #undef _WIN64 */
/* #undef __VISUALC__ */
#ifdef _XBT_WIN32
	#ifndef __GNUC__
/* #undef __GNUC__ */
	#endif
#endif

/* Define to 1 if you have the <unistd.h> header file. */
#define HAVE_UNISTD_H 1
/* Define to 1 if you have the <sys/time.h> header file. */
#define HAVE_SYS_TIME_H 1

/* Define to 1 if mmalloc is compiled in. */
/* #undef HAVE_MMALLOC */

/* Get the config */
#undef SIMGRID_NEED_ASPRINTF
#undef SIMGRID_NEED_VASPRINTF



/* Whether mallocators were enabled in ccmake or not. */
#define MALLOCATOR_COMPILED_IN 1

/* Define if xbt contexts are based on our threads implementation or not */
#define CONTEXT_THREADS 1

/* Jedule output  */
/* #undef HAVE_JEDULE */

/* Tracking of latency bound */
/* #undef HAVE_LATENCY_BOUND_TRACKING */

/* If __thread is available */
#define HAVE_THREAD_LOCAL_STORAGE 1

/* If Model-Checking support was requested */
/* #undef HAVE_MC */

#endif /* SIMGRID_PUBLIC_CONFIG_H */
